<?php

class Controller_Api_V0_StudyPlan extends Controller_Handler_Api {
    private function normalizeDisciplinesData($discipline, $facultyID) {
        if (!isset($discipline->subjectID) && isset($discipline->externalID)) {
            $discipline->subjectID = Model_Subject::withExternalID($discipline->externalID, $discipline->name, '', $facultyID);
        }

        foreach ($discipline->teachers as $teacher) {
            if (empty($teacher->hashSnils)) {
                Log::instance()->add(Log::WARNING, '{0} {1}: {2}', array(
                    '{0}' => 'SYNC_DISCIPLINES',
                    '{1}' => 'Discipline with empty teacher\'s hash INILA',
                    '{2}' => json_encode($discipline),
                ));
            } else {
                $ID = Model_Teacher::withINILA($teacher->hashSnils);
                if ($ID != -1) {
                    $discipline->teacherIDs[] = $ID;
                }
            }
        }

        /*
         * Если нет ни одного преподавателя, то ругаемся в логи и пропускаем дисциплину до лучших времен
         */
        $hasTeachers = !isset($discipline->teachers) || count($discipline->teachers) == 0;
        if (!isset($discipline->teacherIDs) || count($discipline->teacherIDs) == 0) {
            if (!$hasTeachers) {
                Log::instance()->add(Log::WARNING, '{0} {1}: {2}', array(
                    '{0}' => 'SYNC_DISCIPLINES',
                    '{1}' => 'Discipline without teachers',
                    '{2}' => json_encode($discipline),
                ));
            }
        }

        switch ($discipline->type) {
            case 'Экзамен':
                $discipline->type = Model_Discipline::EXAM;
                break;

            case 'Зачет':
                $discipline->type = Model_Discipline::CREDIT;
                break;

            default:
                throw new InvalidArgumentException('Discipline has bad type');
        }

        return $discipline;
    }

    private function processStudyPlan($data, $year, $facultyID) {
        if (!isset($data->id) && isset($data->externalID)) {
            $ID = Model_Plan::withExternalID($data->externalID);
            if ($ID == -1) {
                $data->planModel = Model_Plan::make()
                    ->externalID($data->externalID)
                    ->year($year)
                    ->faculty($facultyID)
                    ->create();
            } else {
                $data->planModel = Model_Plan::load($ID);
            }
        }

        foreach ($data->disciplines as &$discipline) {
            try {
                $discipline = $this->normalizeDisciplinesData($discipline, $facultyID);
            } catch (Exception $e) {
                Log::instance()->add(Log::WARNING, '{0} {1}: {2}', array(
                    '{0}' => 'SYNC_DISCIPLINES',
                    '{1}' => $e->getMessage(),
                    '{2}' => json_encode($discipline),
                ));
                $discipline = null;
            }
        }

        return $data;
    }

    private function processDiscipline($discipline, $semesterID, $facultyID) {
        $disciplineModel = null;
        if (isset($discipline->subjectID)) {
            $disciplineModel = Model_Discipline::find($facultyID, $semesterID, $discipline->subjectID, $discipline->type);
        }

        $lTeacherIDs = [];
        if ($disciplineModel !== null) {
            $disciplineModel->update();

            // Сравнить список пришедших преподавателей и тех, которые уже прикреплены
            if (isset($discipline->teacherIDs)) {
                $lTeacherIDs = array_diff($discipline->teacherIDs, $disciplineModel->getTeachers()->as_array());
            }
        } else {
            if (!isset($discipline->teacherIDs) || count($discipline->teacherIDs) == 0) {
                throw new InvalidArgumentException('Cannot create discipline without teachers');
            }

            $disciplineModel = Model_Discipline::make()
                ->author($discipline->teacherIDs[0])
                ->subject($discipline->subjectID)
                ->semester($semesterID)
                ->type($discipline->type)
                ->faculty($facultyID)
                ->create();

            $lTeacher = $disciplineModel->getTeachers()->as_array();
            foreach ($lTeacher as $teacher) {
                $lTeacherIDs[] = $teacher['ID'];
            }
        }

        foreach ($lTeacherIDs as $teacherID) {
            $teacher = Model_Teacher::with($teacherID);
            $disciplineModel->bind($teacher);
        }

        return $disciplineModel;
    }

    private function processDisciplines($data, $semesterID, $facultyID) {
        $res = [];

        foreach ($data->disciplines as $discipline) {
            if ($discipline == null) continue;

            try {
                $res[] = (int)$this->processDiscipline($discipline, $semesterID, $facultyID)->ID;
            } catch (Exception $e) {
                Log::instance()->add(Log::WARNING, '{0} {1}: {2}', array(
                    '{0}' => 'SYNC_DISCIPLINES',
                    '{1}' => $e->getMessage(),
                    '{2}' => json_encode($discipline),
                ));
            }
        }

        return $res;
    }

    private function processData($data, $requestInfo) {
        $data = $this->processStudyPlan($data, $requestInfo['Year'], $requestInfo['FacultyID']);
        return $this->processDisciplines($data, $requestInfo['SemesterID'], $requestInfo['FacultyID']);
    }

//    /**
//     * @api {put} api/v0/studyPlan Add new study plan(s)
//     * @apiName Add new study plan
//     * @apiGroup Study plans
//     * @apiVersion 0.1.2
//     * @apiParam {String} token Api key
//     * @apiParam {String} [batch] Flag about massive addition
//     * @apiParam {String} externalId
//     * @apiParam {Array} disciplines
//     */
    public function action_put_index() {
        $res = [];

        try {
            if ($this->request->query('batch') !== NULL) {
                $data = json_decode($this->request->body());

                $data->year = (int)$data->year;
                $data->semester = (int)$data->semester;

                $requestInfo = array(
                    'Year' => $data->year,
                    'SemesterID' => (int)Model_Semesters::find($data->year, $data->semester)->ID,
                    'FacultyID' => Model_Faculties::getIdByExternalID($data->faculty),
                );

                foreach ($data->plans as $item) {
                    $res[] = $this->processData($item, $requestInfo);
                }
            } else {
                throw new InvalidArgumentException('No batch data received!');
            }
        } catch (Exception $e) {
            $this->badRequestError($e->getMessage());
        }

        return $res;
    }
}
